#!/bin/bash
#
# Copyright (c) 2024 Alibaba Group Holding Limited
# vtoa is licensed under Mulan PSL v2.
# You can use this software according to the terms and conditions of the Mulan PSL v2.
# You may obtain a copy of Mulan PSL v2 at:
#          http://license.coscl.org.cn/MulanPSL2
# THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
# EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
# MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
# See the Mulan PSL v2 for more details.
#

# used in .spec-%post/%preun, start vtoa by kernel version
# < 5.10-015: do not install
# >= 5.10-017: vtoa-bpf
# >= 5.10-015 && < 5.10-017: vtoa.ko

bpf_start() {
	cgroup_path="/run/vtoa-bpf-cgroup2"
	bpf_obj_dir="/usr/share/vtoa"

	# Because we support modprobe to load vtoa-bpf,
	# restart is necessary to avoid errors caused by multiple 'modprobe'.
	vtoa -a restart -g ${cgroup_path} -b ${bpf_obj_dir}
}

bpf_stop() {
	vtoa -a stop
}

ko_start() {
	# -i: ignore the "install" cmd in /etc/modprobe.d/
	modprobe -i vtoa
}

ko_stop() {
	# -i: ignore the "remove" cmd in /etc/modprobe.d/
	modprobe -ir vtoa
}

# 1 means launch, else remove
mng_cmd=$1

# always do both remove if mng_cmd is 0, because ko and bpf can be launched together by user
ko_manage() {
	if [[ "$mng_cmd" == "1" ]]
	then
		ko_start
	else
		bpf_stop
		ko_stop
	fi
}

bpf_manage() {
	if [[ "$mng_cmd" == "1" ]]
	then
		bpf_start
	else
		bpf_stop
		ko_stop
	fi
}

# version is compared with `sort -V`, e.g., following versions are all larger than $bpf_kernel_version:
# 5.10.134-017, 5.10.135-*, 5.11.*, 6.*
bpf_kernel_version="5.10.134-017"
ko_kernel_version="5.10.134-015"

cur_kernel_ver=$(uname -r)
if [[ "$bpf_kernel_version" == "$(echo -e "$bpf_kernel_version\n$cur_kernel_ver" | sort -V | head -n1)" ]]
then
	# >= 5.10-017
	echo "kernel version >= $bpf_kernel_version: vtoa-bpf."
	bpf_manage

elif [[ "$ko_kernel_version" == "$(echo -e "$ko_kernel_version\n$cur_kernel_ver" | sort -V | head -n1)" ]]
then
	# >= 5.10-015
	echo "kernel version $ko_kernel_version ~ $bpf_kernel_version: vtoa.ko."
	ko_manage

else
	# < 5.10-015
	echo "kernel version < $ko_kernel_version: not support, do nothing."
fi